import { Colors, Validations, Coordinate, Movement, Chessboard } from "./validator";
/**
 * Main class to handle Chess match using light-chess.
 *
 * @remarks
 * Based on `Validator` namespace and its functions.
 */
export declare class Chess {
    /**
     * The default layout for the chessboard.
     */
    static readonly layout: Chessboard;
    /**
     * The current chessboard layout.
     */
    chessboard: Chessboard;
    /**
     * The current player turn.
     */
    turn: Colors;
    /**
     * The winner of the match.
     *
     * @remarks
     * if `null` then the match is not finished.
     */
    checkmate: Colors | null;
    /**
     * Default class constructor.
     *
     * @param layout - The layout of the chessboard at the beginning of the match, if ignored the default layout will be `Chess.layout`. It can be a Chessboard object or a FEN expression.
     */
    constructor(layout?: Chessboard | string);
    /**
     * Executes a move for the current player.
     *
     * @param movement - The movement that will be executed.
     * @returns One of the possible value of `Validations` enumeration.
     */
    move(movement: Movement): Validations;
    /**
     * Evaluates and validates a move within the current chessboard.
     *
     * @param movement - The movement to validate.
     * @returns One of the possible value of `Validations` enumeration.
     *
     * @remarks
     * Non-static wrapper for respective `Validator.validate` function.
     */
    validate: (movement: Movement) => Validations;
    /**
     * Calculates every legal move for a selected piece within the current chessboard.
     *
     * @param coordinate - The coordinate of the piece.
     * @returns An array containing every square's coordinate where the piece can be moved.
     *
     * @remarks
     * Non-static wrapper for respective `Validator.legals` function.
     */
    legals: (coordinate: Coordinate) => Coordinate[];
    /**
     * Calculates every legal move for a selected player within the current chessboard.
     *
     * @param color - The color of the player to calculate the possible moves. If ignored the default player will be the current turn one.
     * @param check - The default value is true. If it is set to false, all the check control will be disabled.
     * @returns An array containing every possible movement.
     *
     * @remarks
     * Non-static wrapper for respective `Validator.possibilities` function.
     */
    possibilities: (color?: Colors, check?: boolean) => Movement[];
    /**
     * Evaluates if the selected player is on check.
     *
     * @param color - The color of the player to evaluate. If ignored the default player will be the current turn one.
     * @returns `true`: The player is on check, `false`: The player is not on check.
     *
     * @remarks
     * Non-static wrapper for respective `Validator.check` function.
     */
    check: (color?: Colors) => boolean;
    /**
     * Creates an UTF-16 string which represents the current chessboard layout.
     *
     * @returns The created UNICODE string.
     *
     * @remarks
     * Non-static wrapper for respective `Chess.unicode` static method.
     */
    unicode: () => string;
    /**
     * Converts a string format coordinate to the respective `Coordinate` object.
     *
     * @param square - The selected square of the chessboard expressed in string format (example: `e2`, `a5`, `h6`).
     * @throws If the string format coordinate is not valid, the method will throw an Error.
     * @returns The respective `Coordinate` object.
     */
    static decode(square: string): Coordinate;
    /**
     * Converts a `Coordinate` object to the respective string format coordinate.
     *
     * @param square - The selected square of the chessboard expressed using a `Coordinate` object.
     * @throws If the `Coordinate` object contains an invalid row or column, the method will throw an Error.
     * @returns The respective string format coordinate.
     */
    static encode(square: Coordinate): string;
    /**
     * Imports a FEN expression format replacing the current match.
     *
     * @param fen - The FEN expression which will be imported (example: `e2`).
     * @throws If the FEN expression is not valid and does not match with the RegExp, the method will throw an Error.
     *
     * @remarks
     * The RegExp pattern used for FEN validation is:
     * `\s*^(((?:[rnbqkpRNBQKP1-8]+\/){7})[rnbqkpRNBQKP1-8]+)\s([b|w])\s([K|Q|k|q|-]{1,4})\s(-|[a-h][1-8])\s(\d+\s\d+)$`
     */
    import(fen: string): void;
    /**
     * Exports the current match in FEN expression format.
     *
     * @returns A string containing the FEN expression.
     */
    export(): string;
    /**
     * Creates an UTF-16 string which represents the current chessboard layout.
     *
     * @param chessboard - The chessboard layout to represent.
     * @returns The created UNICODE string.
     */
    static unicode(chessboard: Chessboard): string;
}
