"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var validator_1 = require("./validator");
/**
 * Main class to handle Chess match using light-chess.
 *
 * @remarks
 * Based on `Validator` namespace and its functions.
 */
var Chess = /** @class */ (function () {
    /**
     * Default class constructor.
     *
     * @param layout - The layout of the chessboard at the beginning of the match, if ignored the default layout will be `Chess.layout`. It can be a Chessboard object or a FEN expression.
     */
    function Chess(layout) {
        var _this = this;
        if (layout === void 0) { layout = Chess.layout; }
        /**
         * The current player turn.
         */
        this.turn = validator_1.Colors.WHITE;
        /**
         * The winner of the match.
         *
         * @remarks
         * if `null` then the match is not finished.
         */
        this.checkmate = null;
        /**
         * Evaluates and validates a move within the current chessboard.
         *
         * @param movement - The movement to validate.
         * @returns One of the possible value of `Validations` enumeration.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.validate` function.
         */
        this.validate = function (movement) {
            return validator_1.Validator.validate(_this.chessboard, movement);
        };
        /**
         * Calculates every legal move for a selected piece within the current chessboard.
         *
         * @param coordinate - The coordinate of the piece.
         * @returns An array containing every square's coordinate where the piece can be moved.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.legals` function.
         */
        this.legals = function (coordinate) {
            return validator_1.Validator.legals(_this.chessboard, coordinate);
        };
        /**
         * Calculates every legal move for a selected player within the current chessboard.
         *
         * @param color - The color of the player to calculate the possible moves. If ignored the default player will be the current turn one.
         * @param check - The default value is true. If it is set to false, all the check control will be disabled.
         * @returns An array containing every possible movement.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.possibilities` function.
         */
        this.possibilities = function (color, check) {
            if (color === void 0) { color = _this.turn; }
            if (check === void 0) { check = true; }
            return validator_1.Validator.possibilities(_this.chessboard, color, check);
        };
        /**
         * Evaluates if the selected player is on check.
         *
         * @param color - The color of the player to evaluate. If ignored the default player will be the current turn one.
         * @returns `true`: The player is on check, `false`: The player is not on check.
         *
         * @remarks
         * Non-static wrapper for respective `Validator.check` function.
         */
        this.check = function (color) {
            if (color === void 0) { color = _this.turn; }
            return validator_1.Validator.check(_this.chessboard, color);
        };
        /**
         * Creates an UTF-16 string which represents the current chessboard layout.
         *
         * @returns The created UNICODE string.
         *
         * @remarks
         * Non-static wrapper for respective `Chess.unicode` static method.
         */
        this.unicode = function () {
            return Chess.unicode(_this.chessboard);
        };
        if (typeof layout == typeof Chess.layout)
            this.chessboard = layout.map(function (row) { return row.slice(); });
        if (typeof layout == "string")
            this.import(layout);
    }
    /**
     * Executes a move for the current player.
     *
     * @param movement - The movement that will be executed.
     * @returns One of the possible value of `Validations` enumeration.
     */
    Chess.prototype.move = function (movement) {
        if (this.checkmate)
            return validator_1.Validations.MATCH_FINISHED;
        var initial = movement.initial, final = movement.final;
        if (this.chessboard[initial.row][initial.column].color != this.turn)
            return validator_1.Validations.ILLEGAL_MOVE;
        var result = this.validate(movement);
        if (result == validator_1.Validations.LEGAL_MOVE) {
            this.chessboard[final.row][final.column] = this.chessboard[initial.row][initial.column];
            this.chessboard[initial.row][initial.column] = validator_1.Pieces.Empty;
            var moves = this.possibilities(this.turn ? validator_1.Colors.WHITE : validator_1.Colors.BLACK);
            if (moves.length == 0) {
                this.checkmate = this.turn;
                return validator_1.Validations.CHECKMATE;
            }
            this.turn = this.turn ? validator_1.Colors.WHITE : validator_1.Colors.BLACK;
            return result;
        }
        return result;
    };
    /**
     * Converts a string format coordinate to the respective `Coordinate` object.
     *
     * @param square - The selected square of the chessboard expressed in string format (example: `e2`, `a5`, `h6`).
     * @throws If the string format coordinate is not valid, the method will throw an Error.
     * @returns The respective `Coordinate` object.
     */
    Chess.decode = function (square) {
        var letters = ["A", "B", "C", "D", "E", "F", "G", "H"];
        var numbers = ["1", "2", "3", "4", "5", "6", "7", "8"];
        var error = new Error("Invalid string format for coordinate, cannot decode it");
        if (square == null || square.length > 2)
            throw error;
        var X = square[0].toUpperCase(), Y = square[1];
        if (letters.indexOf(X) < 0 || numbers.indexOf(Y) < 0)
            throw error;
        var coordinate = {
            row: numbers.indexOf(Y),
            column: letters.indexOf(X)
        };
        return coordinate;
    };
    /**
     * Converts a `Coordinate` object to the respective string format coordinate.
     *
     * @param square - The selected square of the chessboard expressed using a `Coordinate` object.
     * @throws If the `Coordinate` object contains an invalid row or column, the method will throw an Error.
     * @returns The respective string format coordinate.
     */
    Chess.encode = function (square) {
        var letters = ["A", "B", "C", "D", "E", "F", "G", "H"];
        var numbers = ["1", "2", "3", "4", "5", "6", "7", "8"];
        var letter = letters[square.column], number = numbers[square.row];
        if (letter == null || number == null)
            throw new Error("Invalid Coordinate object, the row or the column is not valid, cannot encode it");
        return letter + number;
    };
    /**
     * Imports a FEN expression format replacing the current match.
     *
     * @param fen - The FEN expression which will be imported (example: `e2`).
     * @throws If the FEN expression is not valid and does not match with the RegExp, the method will throw an Error.
     *
     * @remarks
     * The RegExp pattern used for FEN validation is:
     * `\s*^(((?:[rnbqkpRNBQKP1-8]+\/){7})[rnbqkpRNBQKP1-8]+)\s([b|w])\s([K|Q|k|q|-]{1,4})\s(-|[a-h][1-8])\s(\d+\s\d+)$`
     */
    Chess.prototype.import = function (fen) {
        var pattern = /\s*^(((?:[rnbqkpRNBQKP1-8]+\/){7})[rnbqkpRNBQKP1-8]+)\s([b|w])\s([K|Q|k|q|-]{1,4})\s(-|[a-h][1-8])\s(\d+\s\d+)$/;
        if (pattern.test(fen) == false)
            throw new Error("Invalid FEN format string, cannot import it");
        var chessboard = new Array();
        var representation = fen.split(" ")[0].split("/").reverse();
        for (var _i = 0, representation_1 = representation; _i < representation_1.length; _i++) {
            var row = representation_1[_i];
            var line = new Array();
            for (var _a = 0, row_1 = row; _a < row_1.length; _a++) {
                var square = row_1[_a];
                var empty = parseInt(square) || 0;
                if (empty > 0) {
                    for (var i = 0; i < empty; i++)
                        line.push(validator_1.Pieces.Empty);
                    continue;
                }
                var piece = square == square.toUpperCase() ? validator_1.Pieces.White : validator_1.Pieces.Black;
                var letter = square.toUpperCase();
                switch (letter) {
                    case "P":
                        piece = piece.Pawn;
                        break;
                    case "B":
                        piece = piece.Bishop;
                        break;
                    case "N":
                        piece = piece.Knight;
                        break;
                    case "R":
                        piece = piece.Rook;
                        break;
                    case "Q":
                        piece = piece.Queen;
                        break;
                    case "K":
                        piece = piece.King;
                        break;
                }
                line.push(piece);
            }
            chessboard.push(line);
        }
        this.chessboard = chessboard;
        var turn = fen.split(" ")[1].toUpperCase();
        if (turn == "W")
            this.turn = validator_1.Colors.WHITE;
        if (turn == "B")
            this.turn = validator_1.Colors.BLACK;
    };
    /**
     * Exports the current match in FEN expression format.
     *
     * @returns A string containing the FEN expression.
     */
    Chess.prototype.export = function () {
        var representation = new Array();
        var chessboard = this.chessboard.slice().reverse();
        var character = function (piece) {
            var white = ["P", "B", "N", "R", "Q", "K"];
            var black = ["p", "b", "n", "r", "q", "k"];
            return piece.color ? black[piece.type] : white[piece.type];
        };
        for (var _i = 0, chessboard_1 = chessboard; _i < chessboard_1.length; _i++) {
            var row = chessboard_1[_i];
            var line = "";
            var counter = 0;
            for (var index in row) {
                var square = row[index];
                if (square == validator_1.Pieces.Empty) {
                    counter = counter + 1;
                    var finish = parseInt(index) == row.length - 1;
                    if (finish)
                        line = line + counter;
                    continue;
                }
                if (counter > 0)
                    line = line + counter;
                counter = 0;
                line = line + character(square);
            }
            representation.push(line);
        }
        var fen = representation.join("/");
        return fen + (this.turn ? " b " : " w ") + "KQkq - 0 0";
    };
    /**
     * Creates an UTF-16 string which represents the current chessboard layout.
     *
     * @param chessboard - The chessboard layout to represent.
     * @returns The created UNICODE string.
     */
    Chess.unicode = function (chessboard) {
        var emoji = function (piece) {
            if (piece == validator_1.Pieces.Empty)
                return " ";
            var white = ["♙", "♗", "♘", "♖", "♕", "♔"];
            var black = ["♟", "♝", "♞", "♜", "♛", "♚"];
            return piece.color ? black[piece.type] : white[piece.type];
        };
        var ascii = "";
        for (var row = chessboard.length - 1; row >= 0; row--) {
            var append = (row + 1) + " |";
            for (var column in chessboard[row])
                append = append + emoji(chessboard[row][column]) + " |";
            ascii = ascii + append + "\n";
        }
        ascii = ascii + ("   A  B  C  D  E  F  G  H");
        return ascii;
    };
    /**
     * The default layout for the chessboard.
     */
    Chess.layout = [
        [validator_1.Pieces.White.Rook, validator_1.Pieces.White.Knight, validator_1.Pieces.White.Bishop, validator_1.Pieces.White.Queen, validator_1.Pieces.White.King, validator_1.Pieces.White.Bishop, validator_1.Pieces.White.Knight, validator_1.Pieces.White.Rook],
        [validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn, validator_1.Pieces.White.Pawn],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty, validator_1.Pieces.Empty],
        [validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn, validator_1.Pieces.Black.Pawn],
        [validator_1.Pieces.Black.Rook, validator_1.Pieces.Black.Knight, validator_1.Pieces.Black.Bishop, validator_1.Pieces.Black.Queen, validator_1.Pieces.Black.King, validator_1.Pieces.Black.Bishop, validator_1.Pieces.Black.Knight, validator_1.Pieces.Black.Rook],
    ];
    return Chess;
}());
exports.Chess = Chess;
//# sourceMappingURL=chess.js.map