/**
 * Enumerator which represents the two possible piece color in chess.
 */
export declare enum Colors {
    WHITE = 0,
    BLACK = 1
}
/**
 * Enumerator which represents the six possible types of piece in chess.
 */
export declare enum Types {
    PAWN = 0,
    BISHOP = 1,
    KNIGHT = 2,
    ROOK = 3,
    QUEEN = 4,
    KING = 5
}
/**
 * Enumerator which represents one of the possible result given by the `Validator.validate` function.
 * Each value of the enumeration indicates the state occured.
 *
 * @param LEGAL_MOVE - The move is legal.
 * @param ILLEGAL_MOVE - The move is illegal for the selected piece move pattern.
 * @param KING_ON_CHECK - The king of the player who tried to validate the move is on check.
 * @param BLANK_SQUARE - The selected square is empty and does not contain any piece to move.
 * @param OUT_OF_BOARD - The selected square is out of the board.
 * @param CHECKMATE - The executed move checkmated the opponent.
 * @param MATCH_FINISHED - The move cannot be executed because the match is finished.
 */
export declare enum Validations {
    LEGAL_MOVE = 0,
    ILLEGAL_MOVE = 1,
    KING_ON_CHECK = 2,
    BLANK_SQUARE = 3,
    OUT_OF_BOARD = 4,
    CHECKMATE = 5,
    MATCH_FINISHED = 6
}
/**
 * Type-Alias which represents a coordinate which points to one of the chessboard's square.
 */
export declare type Coordinate = {
    row: number;
    column: number;
};
/**
 * Type-Alias which represents a movement.
 */
export declare type Movement = {
    initial: Coordinate;
    final: Coordinate;
};
/**
 * Type-Alias which represents a chessboard row bone and type.
 */
export declare type Row = [Square, Square, Square, Square, Square, Square, Square, Square];
/**
 * Type-Alias which represents the exact chessboard bone and type.
 */
export declare type Chessboard = [[Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square], [Square, Square, Square, Square, Square, Square, Square, Square]];
/**
 * Type-Alias which represents both short and long castling status for white and black player.
 */
export declare type Castling = {
    white: {
        short: boolean;
        long: boolean;
    };
    black: {
        short: boolean;
        long: boolean;
    };
};
/**
 * Core of the chess-engine, contains useful functions to evaluates moves and matches.
 */
export declare namespace Validator {
    /**
     * Evaluates and validates a move within the current chessboard.
     *
     * @param chessboard - The chessboard layout to use.
     * @param movement - The movement to validate.
     * @returns One of the possible value of `Validations` enumeration.
     */
    function validate(chessboard: Chessboard, movement: Movement): Validations;
    /**
     * Calculates every legal move for a selected piece within the current chessboard.
     *
     * @param chessboard - The chessboard layout to use.
     * @param coordinate - The coordinate of the piece.
     * @returns An array containing every square's coordinate where the piece can be moved.
     */
    function legals(chessboard: Chessboard, coordinate: Coordinate): Coordinate[];
    /**
     * Calculates every legal move for a selected player within the current chessboard.
     *
     * @param chessboard - The chessboard layout to use.
     * @param color - The color of the player to calculate the possible moves. If ignored the default player will be the current turn one.
     * @param check - The default value is true. If it is set to false, all the check control will be disabled.
     * @returns An array containing every possible movement.
     */
    function possibilities(chessboard: Chessboard, color: Colors, check?: boolean): Movement[];
    /**
     * Evaluates if the selected player is on check.
     *
     * @param chessboard - The chessboard layout to use.
     * @param color - The color of the player to evaluate. If ignored the default player will be the current turn one.
     * @returns `true`: The player is on check, `false`: The player is not on check.
     */
    function check(chessboard: Chessboard, color: Colors): boolean;
}
/**
 * Class which represents a chessboard square.
 */
export declare class Square {
    /**
     * The color of the piece on the square.
     *
     * @remarks If it is set to `null` then the square will be empty.
     */
    color: Colors | null;
    /**
     * The type of the piece on the square.
     *
     * @remarks If it is set to `null` then the square will be empty.
     */
    type: Types | null;
    /**
     * The validation function of the piece on the square.
     *
     * @remarks If it is set to `null` then the square will be empty.
     */
    validation: Function | null;
    /**
     * Default class constructor.
     *
     * @param color - The color of the piece on the square.
     * @param type - The type of the piece on the square.
     * @param validation - The validation function of the piece on the square.
     */
    constructor(color?: Colors | null, type?: Types | null, validation?: Function | null);
}
/**
 * Wrapper object to represent every possible chess piece, included an empty square.
 */
export declare const Pieces: {
    White: {
        Pawn: Square;
        Bishop: Square;
        Knight: Square;
        Rook: Square;
        Queen: Square;
        King: Square;
    };
    Black: {
        Pawn: Square;
        Bishop: Square;
        Knight: Square;
        Rook: Square;
        Queen: Square;
        King: Square;
    };
    Empty: Square;
};
