"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Enumerator which represents the two possible piece color in chess.
 */
var Colors;
(function (Colors) {
    Colors[Colors["WHITE"] = 0] = "WHITE";
    Colors[Colors["BLACK"] = 1] = "BLACK";
})(Colors = exports.Colors || (exports.Colors = {}));
/**
 * Enumerator which represents the six possible types of piece in chess.
 */
var Types;
(function (Types) {
    Types[Types["PAWN"] = 0] = "PAWN";
    Types[Types["BISHOP"] = 1] = "BISHOP";
    Types[Types["KNIGHT"] = 2] = "KNIGHT";
    Types[Types["ROOK"] = 3] = "ROOK";
    Types[Types["QUEEN"] = 4] = "QUEEN";
    Types[Types["KING"] = 5] = "KING";
})(Types = exports.Types || (exports.Types = {}));
/**
 * Enumerator which represents one of the possible result given by the `Validator.validate` function.
 * Each value of the enumeration indicates the state occured.
 *
 * @param LEGAL_MOVE - The move is legal.
 * @param ILLEGAL_MOVE - The move is illegal for the selected piece move pattern.
 * @param KING_ON_CHECK - The king of the player who tried to validate the move is on check.
 * @param BLANK_SQUARE - The selected square is empty and does not contain any piece to move.
 * @param OUT_OF_BOARD - The selected square is out of the board.
 * @param CHECKMATE - The executed move checkmated the opponent.
 * @param MATCH_FINISHED - The move cannot be executed because the match is finished.
 */
var Validations;
(function (Validations) {
    Validations[Validations["LEGAL_MOVE"] = 0] = "LEGAL_MOVE";
    Validations[Validations["ILLEGAL_MOVE"] = 1] = "ILLEGAL_MOVE";
    Validations[Validations["KING_ON_CHECK"] = 2] = "KING_ON_CHECK";
    Validations[Validations["BLANK_SQUARE"] = 3] = "BLANK_SQUARE";
    Validations[Validations["OUT_OF_BOARD"] = 4] = "OUT_OF_BOARD";
    Validations[Validations["CHECKMATE"] = 5] = "CHECKMATE";
    Validations[Validations["MATCH_FINISHED"] = 6] = "MATCH_FINISHED";
})(Validations = exports.Validations || (exports.Validations = {}));
/**
 * Core of the chess-engine, contains useful functions to evaluates moves and matches.
 */
var Validator;
(function (Validator) {
    /**
     * Evaluates and validates a move within the current chessboard.
     *
     * @param chessboard - The chessboard layout to use.
     * @param movement - The movement to validate.
     * @returns One of the possible value of `Validations` enumeration.
     */
    function validate(chessboard, movement) {
        var _a;
        var square = (_a = chessboard[movement.initial.row]) === null || _a === void 0 ? void 0 : _a[movement.initial.column];
        if (square == null)
            return Validations.OUT_OF_BOARD;
        if (square == exports.Pieces.Empty)
            return Validations.BLANK_SQUARE;
        var moves = square.validation(chessboard, movement.initial, false);
        var filters = Evaluates.filter(chessboard, movement.initial, moves);
        for (var _i = 0, filters_1 = filters; _i < filters_1.length; _i++) {
            var filter = filters_1[_i];
            if (filter.row == movement.final.row && filter.column == movement.final.column)
                return Validations.LEGAL_MOVE;
        }
        for (var _b = 0, moves_1 = moves; _b < moves_1.length; _b++) {
            var move = moves_1[_b];
            if (move.row == movement.final.row && move.column == movement.final.column)
                return Validations.KING_ON_CHECK;
        }
        return Validations.ILLEGAL_MOVE;
    }
    Validator.validate = validate;
    /**
     * Calculates every legal move for a selected piece within the current chessboard.
     *
     * @param chessboard - The chessboard layout to use.
     * @param coordinate - The coordinate of the piece.
     * @returns An array containing every square's coordinate where the piece can be moved.
     */
    function legals(chessboard, coordinate) {
        var _a, _b;
        var square = (_a = chessboard[coordinate.row]) === null || _a === void 0 ? void 0 : _a[coordinate.column];
        var coordinates = ((_b = square === null || square === void 0 ? void 0 : square.validation) === null || _b === void 0 ? void 0 : _b.call(square, chessboard, coordinate, true)) || new Array();
        return coordinates;
    }
    Validator.legals = legals;
    /**
     * Calculates every legal move for a selected player within the current chessboard.
     *
     * @param chessboard - The chessboard layout to use.
     * @param color - The color of the player to calculate the possible moves. If ignored the default player will be the current turn one.
     * @param check - The default value is true. If it is set to false, all the check control will be disabled.
     * @returns An array containing every possible movement.
     */
    function possibilities(chessboard, color, check) {
        if (check === void 0) { check = true; }
        var movements = new Array();
        for (var row in chessboard)
            for (var column in chessboard[row]) {
                if (chessboard[row][column].color != color)
                    continue;
                var initial = { row: +row, column: +column };
                var finals = chessboard[row][column].validation(chessboard, initial, check);
                for (var _i = 0, finals_1 = finals; _i < finals_1.length; _i++) {
                    var final = finals_1[_i];
                    movements.push({ initial: initial, final: final });
                }
            }
        return movements;
    }
    Validator.possibilities = possibilities;
    /**
     * Evaluates if the selected player is on check.
     *
     * @param chessboard - The chessboard layout to use.
     * @param color - The color of the player to evaluate. If ignored the default player will be the current turn one.
     * @returns `true`: The player is on check, `false`: The player is not on check.
     */
    function check(chessboard, color) {
        var piece = color ? exports.Pieces.Black.King : exports.Pieces.White.King;
        var king = null;
        for (var row in chessboard)
            for (var column in chessboard[row])
                if (chessboard[row][column] == piece)
                    king = { row: +row, column: +column };
        var possibilities = Validator.possibilities(chessboard, color ? Colors.WHITE : Colors.BLACK, false);
        for (var _i = 0, possibilities_1 = possibilities; _i < possibilities_1.length; _i++) {
            var possibility = possibilities_1[_i];
            if (possibility.final.row == king.row && possibility.final.column == king.column)
                return true;
        }
        return false;
    }
    Validator.check = check;
})(Validator = exports.Validator || (exports.Validator = {}));
var Evaluates;
(function (Evaluates) {
    function pawn(chessboard, coordinate, check) {
        var _a, _b, _c, _d, _e, _f, _g;
        var coordinates = new Array();
        var row = coordinate.row, column = coordinate.column;
        var addictive = chessboard[row][column].color ? Additives.NEGATIVE : Additives.POSITIVE;
        if (((_a = chessboard[row + addictive]) === null || _a === void 0 ? void 0 : _a[column]) == exports.Pieces.Empty)
            coordinates.push({ row: row + addictive, column: column });
        var eat = chessboard[row][column].color ? Colors.WHITE : Colors.BLACK;
        if (((_c = (_b = chessboard[row + addictive]) === null || _b === void 0 ? void 0 : _b[column + 1]) === null || _c === void 0 ? void 0 : _c.color) == eat)
            coordinates.push({ row: row + addictive, column: column + 1 });
        if (((_e = (_d = chessboard[row + addictive]) === null || _d === void 0 ? void 0 : _d[column - 1]) === null || _e === void 0 ? void 0 : _e.color) == eat)
            coordinates.push({ row: row + addictive, column: column - 1 });
        if (row == 6 || row == 1)
            if (((_f = chessboard[row + addictive]) === null || _f === void 0 ? void 0 : _f[column]) == exports.Pieces.Empty)
                if (((_g = chessboard[row + addictive + addictive]) === null || _g === void 0 ? void 0 : _g[column]) == exports.Pieces.Empty)
                    coordinates.push({ row: row + addictive + addictive, column: column });
        return check ? Evaluates.filter(chessboard, coordinate, coordinates) : coordinates;
    }
    Evaluates.pawn = pawn;
    function bishop(chessboard, coordinate, check) {
        var coordinates = new Array();
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.POSITIVE, column: Additives.POSITIVE }));
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.POSITIVE, column: Additives.NEGATIVE }));
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.NEGATIVE, column: Additives.POSITIVE }));
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.NEGATIVE, column: Additives.NEGATIVE }));
        return check ? Evaluates.filter(chessboard, coordinate, coordinates) : coordinates;
    }
    Evaluates.bishop = bishop;
    function knight(chessboard, coordinate, check) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z;
        var coordinates = new Array();
        var row = coordinate.row, column = coordinate.column;
        var eat = chessboard[row][column].color ? Colors.WHITE : Colors.BLACK;
        if (((_a = chessboard[row + 2]) === null || _a === void 0 ? void 0 : _a[column - 1]) == exports.Pieces.Empty || ((_c = (_b = chessboard[row + 2]) === null || _b === void 0 ? void 0 : _b[column - 1]) === null || _c === void 0 ? void 0 : _c.color) == eat)
            coordinates.push({ row: row + 2, column: column - 1 });
        if (((_d = chessboard[row + 2]) === null || _d === void 0 ? void 0 : _d[column + 1]) == exports.Pieces.Empty || ((_f = (_e = chessboard[row + 2]) === null || _e === void 0 ? void 0 : _e[column - 1]) === null || _f === void 0 ? void 0 : _f.color) == eat)
            coordinates.push({ row: row + 2, column: column + 1 });
        if (((_g = chessboard[row - 2]) === null || _g === void 0 ? void 0 : _g[column + 1]) == exports.Pieces.Empty || ((_j = (_h = chessboard[row + 2]) === null || _h === void 0 ? void 0 : _h[column - 1]) === null || _j === void 0 ? void 0 : _j.color) == eat)
            coordinates.push({ row: row - 2, column: column + 1 });
        if (((_k = chessboard[row - 2]) === null || _k === void 0 ? void 0 : _k[column - 1]) == exports.Pieces.Empty || ((_m = (_l = chessboard[row + 2]) === null || _l === void 0 ? void 0 : _l[column - 1]) === null || _m === void 0 ? void 0 : _m.color) == eat)
            coordinates.push({ row: row - 2, column: column - 1 });
        if (((_o = chessboard[row + 1]) === null || _o === void 0 ? void 0 : _o[column - 2]) == exports.Pieces.Empty || ((_q = (_p = chessboard[row + 2]) === null || _p === void 0 ? void 0 : _p[column - 1]) === null || _q === void 0 ? void 0 : _q.color) == eat)
            coordinates.push({ row: row + 1, column: column - 2 });
        if (((_r = chessboard[row + 1]) === null || _r === void 0 ? void 0 : _r[column + 2]) == exports.Pieces.Empty || ((_t = (_s = chessboard[row + 2]) === null || _s === void 0 ? void 0 : _s[column - 1]) === null || _t === void 0 ? void 0 : _t.color) == eat)
            coordinates.push({ row: row + 1, column: column + 2 });
        if (((_u = chessboard[row - 1]) === null || _u === void 0 ? void 0 : _u[column + 2]) == exports.Pieces.Empty || ((_w = (_v = chessboard[row + 2]) === null || _v === void 0 ? void 0 : _v[column - 1]) === null || _w === void 0 ? void 0 : _w.color) == eat)
            coordinates.push({ row: row - 1, column: column + 2 });
        if (((_x = chessboard[row - 1]) === null || _x === void 0 ? void 0 : _x[column - 2]) == exports.Pieces.Empty || ((_z = (_y = chessboard[row + 2]) === null || _y === void 0 ? void 0 : _y[column - 1]) === null || _z === void 0 ? void 0 : _z.color) == eat)
            coordinates.push({ row: row - 1, column: column - 2 });
        return check ? Evaluates.filter(chessboard, coordinate, coordinates) : coordinates;
    }
    Evaluates.knight = knight;
    function rook(chessboard, coordinate, check) {
        var coordinates = new Array();
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.POSITIVE, column: Additives.NEUTRAL }));
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.NEUTRAL, column: Additives.POSITIVE }));
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.NEGATIVE, column: Additives.NEUTRAL }));
        coordinates.push.apply(coordinates, Evaluates.linear(chessboard, coordinate, { row: Additives.NEUTRAL, column: Additives.NEGATIVE }));
        return check ? Evaluates.filter(chessboard, coordinate, coordinates) : coordinates;
    }
    Evaluates.rook = rook;
    function queen(chessboard, coordinate, check) {
        var coordinates = new Array();
        var bishop = Evaluates.bishop(chessboard, coordinate, check), rook = Evaluates.rook(chessboard, coordinate, check);
        coordinates.push.apply(coordinates, bishop);
        coordinates.push.apply(coordinates, rook);
        return check ? Evaluates.filter(chessboard, coordinate, coordinates) : coordinates;
    }
    Evaluates.queen = queen;
    function king(chessboard, coordinate, check) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z;
        var coordinates = new Array();
        var row = coordinate.row, column = coordinate.column;
        var eat = chessboard[row][column].color ? Colors.WHITE : Colors.BLACK;
        if (((_a = chessboard[row + 1]) === null || _a === void 0 ? void 0 : _a[column]) == exports.Pieces.Empty || ((_c = (_b = chessboard[row + 2]) === null || _b === void 0 ? void 0 : _b[column - 1]) === null || _c === void 0 ? void 0 : _c.color) == eat)
            coordinates.push({ row: row + 1, column: column });
        if (((_d = chessboard[row - 1]) === null || _d === void 0 ? void 0 : _d[column]) == exports.Pieces.Empty || ((_f = (_e = chessboard[row + 2]) === null || _e === void 0 ? void 0 : _e[column - 1]) === null || _f === void 0 ? void 0 : _f.color) == eat)
            coordinates.push({ row: row - 1, column: column });
        if (((_g = chessboard[row]) === null || _g === void 0 ? void 0 : _g[column + 1]) == exports.Pieces.Empty || ((_j = (_h = chessboard[row + 2]) === null || _h === void 0 ? void 0 : _h[column - 1]) === null || _j === void 0 ? void 0 : _j.color) == eat)
            coordinates.push({ row: row, column: column + 1 });
        if (((_k = chessboard[row]) === null || _k === void 0 ? void 0 : _k[column - 1]) == exports.Pieces.Empty || ((_m = (_l = chessboard[row + 2]) === null || _l === void 0 ? void 0 : _l[column - 1]) === null || _m === void 0 ? void 0 : _m.color) == eat)
            coordinates.push({ row: row, column: column - 1 });
        if (((_o = chessboard[row + 1]) === null || _o === void 0 ? void 0 : _o[column - 1]) == exports.Pieces.Empty || ((_q = (_p = chessboard[row + 2]) === null || _p === void 0 ? void 0 : _p[column - 1]) === null || _q === void 0 ? void 0 : _q.color) == eat)
            coordinates.push({ row: row + 1, column: column - 1 });
        if (((_r = chessboard[row + 1]) === null || _r === void 0 ? void 0 : _r[column + 1]) == exports.Pieces.Empty || ((_t = (_s = chessboard[row + 2]) === null || _s === void 0 ? void 0 : _s[column - 1]) === null || _t === void 0 ? void 0 : _t.color) == eat)
            coordinates.push({ row: row + 1, column: column + 1 });
        if (((_u = chessboard[row - 1]) === null || _u === void 0 ? void 0 : _u[column + 1]) == exports.Pieces.Empty || ((_w = (_v = chessboard[row + 2]) === null || _v === void 0 ? void 0 : _v[column - 1]) === null || _w === void 0 ? void 0 : _w.color) == eat)
            coordinates.push({ row: row - 1, column: column + 1 });
        if (((_x = chessboard[row - 1]) === null || _x === void 0 ? void 0 : _x[column - 1]) == exports.Pieces.Empty || ((_z = (_y = chessboard[row + 2]) === null || _y === void 0 ? void 0 : _y[column - 1]) === null || _z === void 0 ? void 0 : _z.color) == eat)
            coordinates.push({ row: row - 1, column: column - 1 });
        return check ? Evaluates.filter(chessboard, coordinate, coordinates) : coordinates;
    }
    Evaluates.king = king;
    function filter(chessboard, coordinate, possibilities) {
        var filtered = new Array();
        for (var _i = 0, possibilities_2 = possibilities; _i < possibilities_2.length; _i++) {
            var possibility = possibilities_2[_i];
            var temp = chessboard.map(function (row) { return row.slice(); });
            temp[possibility.row][possibility.column] = temp[coordinate.row][coordinate.column];
            temp[coordinate.row][coordinate.column] = exports.Pieces.Empty;
            var check = Validator.check(temp, chessboard[coordinate.row][coordinate.column].color);
            if (check == false)
                filtered.push(possibility);
        }
        return filtered;
    }
    Evaluates.filter = filter;
    function linear(chessboard, coordinate, additive) {
        var _a;
        var coordinates = new Array();
        var row = coordinate.row, column = coordinate.column;
        for (var i = 1; i < chessboard.length; i++) {
            if (((_a = chessboard[row + i * additive.row]) === null || _a === void 0 ? void 0 : _a[column + i * additive.column]) == undefined)
                break;
            var square = chessboard[row + i * additive.row][column + i * additive.column];
            if (square.color == chessboard[row][column].color)
                break;
            coordinates.push({ row: row + i * additive.row, column: column + i * additive.column });
            if (square == exports.Pieces.Empty)
                continue;
            break;
        }
        return coordinates;
    }
    Evaluates.linear = linear;
    var Additives;
    (function (Additives) {
        Additives[Additives["NEGATIVE"] = -1] = "NEGATIVE";
        Additives[Additives["NEUTRAL"] = 0] = "NEUTRAL";
        Additives[Additives["POSITIVE"] = 1] = "POSITIVE";
    })(Additives || (Additives = {}));
})(Evaluates || (Evaluates = {}));
/**
 * Class which represents a chessboard square.
 */
var Square = /** @class */ (function () {
    /**
     * Default class constructor.
     *
     * @param color - The color of the piece on the square.
     * @param type - The type of the piece on the square.
     * @param validation - The validation function of the piece on the square.
     */
    function Square(color, type, validation) {
        if (color === void 0) { color = null; }
        if (type === void 0) { type = null; }
        if (validation === void 0) { validation = null; }
        this.color = color;
        this.type = type;
        this.validation = validation;
    }
    return Square;
}());
exports.Square = Square;
/**
 * Wrapper object to represent every possible chess piece, included an empty square.
 */
exports.Pieces = {
    White: {
        Pawn: new Square(Colors.WHITE, Types.PAWN, Evaluates.pawn),
        Bishop: new Square(Colors.WHITE, Types.BISHOP, Evaluates.bishop),
        Knight: new Square(Colors.WHITE, Types.KNIGHT, Evaluates.knight),
        Rook: new Square(Colors.WHITE, Types.ROOK, Evaluates.rook),
        Queen: new Square(Colors.WHITE, Types.QUEEN, Evaluates.queen),
        King: new Square(Colors.WHITE, Types.KING, Evaluates.king),
    },
    Black: {
        Pawn: new Square(Colors.BLACK, Types.PAWN, Evaluates.pawn),
        Bishop: new Square(Colors.BLACK, Types.BISHOP, Evaluates.bishop),
        Knight: new Square(Colors.BLACK, Types.KNIGHT, Evaluates.knight),
        Rook: new Square(Colors.BLACK, Types.ROOK, Evaluates.rook),
        Queen: new Square(Colors.BLACK, Types.QUEEN, Evaluates.queen),
        King: new Square(Colors.BLACK, Types.KING, Evaluates.king),
    },
    Empty: new Square()
};
//# sourceMappingURL=validator.js.map